Package: bdsmatrix
Title: Routines for Block Diagonal Symmetric Matrices
Maintainer: Terry Therneau <therneau.terry@mayo.edu>
Version: 1.3-7
Date: 2024-03-01
Depends: methods, R (>= 2.0.0)
LazyLoad: Yes
Author: Terry Therneau
Description: This is a special case of sparse matrices, used by coxme. 
License: LGPL-2
Collate: bdsmatrix.R gchol.R gchol.bdsmatrix.R as.matrix.bdsmatrix.R
        bdsBlock.R bdsI.R bdsmatrix.ibd.R bdsmatrix.reconcile.R
        diag.bdsmatrix.R listbdsmatrix.R multiply.bdsmatrix.R
        solve.bdsmatrix.R solve.gchol.R solve.gchol.bdsmatrix.R
        backsolve.R
NeedsCompilation: yes
Packaged: 2024-03-01 12:06:55 UTC; therneau
Repository: CRAN
Date/Publication: 2024-03-02 12:32:36 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 06:25:13 UTC; unix
