Package: bdsmatrix
Title: Routines for Block Diagonal Symmetric Matrices
Maintainer: Terry Therneau <therneau.terry@mayo.edu>
Version: 1.3-6
Date: 2022-06-03
Depends: methods, R (>= 2.0.0)
LazyLoad: Yes
Author: Terry Therneau
Description: This is a special case of sparse matrices, used by coxme. 
License: LGPL-2
Collate: bdsmatrix.R gchol.R gchol.bdsmatrix.R as.matrix.bdsmatrix.R
        bdsBlock.R bdsI.R bdsmatrix.ibd.R bdsmatrix.reconcile.R
        diag.bdsmatrix.R listbdsmatrix.R multiply.bdsmatrix.R
        solve.bdsmatrix.R solve.gchol.R solve.gchol.bdsmatrix.R
        backsolve.R
NeedsCompilation: yes
Packaged: 2022-06-03 11:33:01 UTC; therneau
Repository: CRAN
Date/Publication: 2022-06-03 22:00:02 UTC
