useDynLib(bdsmatrix, .registration=TRUE)
# Currently group generics have to be explicitly imported (Bug?)
#importFrom("methods", Ops, Arith, Math, Math2)
import("methods")
importFrom("stats", "median")
export("bdsmatrix", "gchol", "bdsmatrix.reconcile", "bdsmatrix.ibd",
	"listbdsmatrix", "bdsBlock", "bdsI")
exportClasses("bdsmatrix", "gchol", "gchol.bdsmatrix")
exportMethods("Math", "Math2", "Ops", 
              "%*%", "gchol", "min", "max", "range", "any",
              "all", "sum", "prod", "diag", "diag<-",
              "backsolve")
S3method(as.matrix, bdsmatrix)
S3method(as.matrix, gchol)
S3method(as.matrix, gchol.bdsmatrix)
S3method(print, bdsmatrix)
S3method(solve, gchol)
S3method(solve, bdsmatrix)
S3method(solve, gchol.bdsmatrix)
S3method(unique, bdsmatrix)
